@@ -3,7 +3,7 @@
 #
 #     Spatstat options and other internal states
 #
-#    $Revision: 1.90 $   $Date: 2020/11/30 13:08:13 $
+#    $Revision: 1.91 $   $Date: 2021/07/09 05:53:14 $
 #
 #
 
@@ -505,6 +505,12 @@ warn.once <- function(key, ...) {
          check=function(x) { is.logical(x) && length(x) == 1 },
          valid="a single logical value"
        ),
+       check.nleqslv.loaded=list(
+         # developmental
+         default=TRUE,
+         check=function(x) { is.logical(x) && length(x) == 1 },
+         valid="a single logical value"
+       ),
        Clinequad = list(
          # use C code for 'linequad'
          default=TRUE,
