@@ -1,7 +1,7 @@
 #
 #  psp.R
 #
-#  $Revision: 1.109 $ $Date: 2020/12/19 05:25:06 $
+#  $Revision: 1.110 $ $Date: 2022/01/04 05:30:06 $
 #
 # Class "psp" of planar line segment patterns
 #
@@ -201,7 +201,7 @@ as.psp.default <- function(x, ..., window=NULL, marks=NULL,
 
 as.psp.owin <- function(x, ..., window=NULL,
                         check=spatstat.options("checksegments"), fatal=TRUE) {
-  .Deprecated("edges", package="spatstat")
+  .Deprecated("edges", package="spatstat.geom")
   edges(x, ..., window=window, check=check)
 }
 
